
/* TSTDATE.C Uses jul_day to produce the astronomical Julian day number
              for a given year, month, and fractional day; additional
              date routines are also included. */
#include <conio.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <stdarg.h>
#include <ctype.h>
#include <dos.h>
#include "date.h"

#define  LF  -75
#define  RT  -77
#define  BK   8
#define  CR   13
#define  ESC  27

char *month_name[] = {"","January","February","March","April","May","June",
                "July","August","September","October","November","December"};
char *day_name[] = {    /* for julian day % 7 */
 "Tuesday","Wednesday","Thursday","Friday","Saturday","Sunday","Monday"};

int read_date(char *s, int *month, double *day, int *year);
int initialize(int *col, int num);  /* column positions for menu items */
void high(int current);             /* print 'current' item in high attr */
void norm(int current);             /* print 'current' item in norm attr */
void do_fcn(int current);           /* carry out the process indicated */
int valid(int m, int d, int y);     /* check validity of date */
void std_screen();                  /* normal entry screen */
void menu_bar(int col[], int n);    /* show menu items on top line */
void normal_exit();
extern                              /* all in module utils.c */
  void showcursor();
  void hidecursor();
  void rect(int left, int top, int right, int bottom, int attr, int border);
  int getstr(char *s);   /* replacement for gets(s) */
  int getdouble(char *s, ... );

int norm_attr = 16*1+15;            /* for entry  screen */
int high_attr = 16*6+7;             /* for highlighting menu items */
int bar_attr = 16*7;                /* for menu bar */
int old_attr;                       /* text attribute at start of program */
int mono=0;                         /* flag for mono system */
int left,top,right,bottom;          /* for text entry box */

/* menu items */
char *msg[] = {" Julian ",
               " JDtoMDY ",
               " Diffdate ",
               " DOY ",
               " DOYtoMD ",
               " Weekday ",
               " Easter "};

int row = 1;                /* starting row (in window) for input */
int num = 7;                /* number of menu items */
int current = 0;            /* current menu item */
int col[7];                 /* positions of menu items */

main()
{
  struct text_info oldtext; /* for saving original text screen values */

  gettextinfo(&oldtext);     /* get original text settings */
  old_attr = oldtext.normattr;    /* save old screen attribute */
  if (oldtext.currmode == 7) {
    mono = 1;
    norm_attr = bar_attr = 7;
    high_attr = 16*7;
  }
  left = 10; top = 2; right = 70; bottom = 25;  /* for entry box */
  textattr(7);
  clrscr();
  std_screen();              /* normal entry screen */
  while(1) {
    hidecursor();
    high(current);           /* highlight current item */
    switch(getkey()) {
      case LF:               /* move left in menu */
        if (current > 0)
          norm(current--);
        break;
      case RT:               /* move right in menu */
        if (current < num)
          norm(current++);
        break;
      case CR:               /* do current function */
        do_fcn(current);
        break;
      case ESC:              /* exit */
        normal_exit();
    }
  }
}

/*---------------------------------------------------------------------------*/
void do_fcn(int current)
{
  double d;                    /* input day */
  int m,y;                     /* input month,year */
  int d_int;                   /* integer part of day */
  double d1;                   /* second day for diffdate() */
  int m1,y1;                   /* second month,year for diffdate */
  int d1_int;                  /* integer part of day1 */
  int wk;                      /* day of week */
  double jul,jul1;             /* Julian day numbers */
  int day_num,MaxDOY;          /* day of year, maximum day of year */
  char c,s[80];                /* for user input */
  long J;
  textattr(norm_attr);
  showcursor();
  if (row >=22) {
    row = 1;
    clrscr();
    std_screen();
    high(current);
    textattr(norm_attr);
    showcursor();
  }
  gotoxy(2,row++);
  switch(current) {
    case 0:             /* find Julian day number of input date */
      cprintf("Enter month, day, year: "); getstr(s);
      if (!s[0])
        break;
      gotoxy(2,row++);
      if (!read_date(s,&m,&d,&y)) {
        cprintf("Invalid entry");
        break;
      }
      d_int = (int)d;
      if (!valid(m, d_int, y)) {
        cprintf("Invalid date");
        break;
      }
      else {
        jul = julian_day(m,d,y);
        cprintf("Julian day number = %.3f\n",jul);
        break;
      }
    case 1:            /* find date for given Julian day number */
      cprintf("Enter Julian day number: "); getstr(s);
      if (!s[0])
        break;
      jul = atof(s);
      gotoxy(2,row++);
      if (jul < 0 || jul > julian_day(1,1.5,MAXYEAR))
        cprintf("Invalid value");
      else {
        JDtoMDY(jul,&m,&d,&y);  /* set m,d,y for given Julian day number */
        cprintf("%s %.3f, %d\n",month_name[m],d,y);
      }
      break;
    case 2:                  /* find difference between two dates */
      cprintf("Enter initial month/day/year: "); getstr(s);
      if (!s[0])
        break;
      if (!read_date(s,&m,&d,&y))
        break;
      d_int = (int)d;
      gotoxy(2,row++);
      if(!valid(m, d_int, y)) {
        cprintf("Invalid date");
        break;
      }
      cprintf("Enter second month/day/year: "); getstr(s);
      if (!s[0])
        break;
      if (!read_date(s,&m1,&d1,&y1))
        break;
      d1_int = (int)d1;
      gotoxy(2,row++);
      if(!valid(m1, d1_int, y1)) {
        cprintf("Invalid date");
        break;
      }
      jul = julian_day(m,d,y);
      jul1 = julian_day(m1,d1,y1);
      cprintf("Difference = %.2f days\n",(jul1 - jul));
      break;
    case 3:               /* find day of year for given month, day, year */
      cprintf("Enter month, day, year: "); getstr(s);
      if (!s[0])
        break;
      if (!read_date(s,&m,&d,&y))
        break;
      gotoxy(2,row++);
      d_int = (int)d;
      if(!valid(m, d_int, y)) {
        cprintf("Invalid date");
        break;
      }
      else
        cprintf("Day-of-Year = %d\n",day_of_year(m,d_int,y));
      break;
    case 4:         /* find month and day for given year and day of year */
      cprintf("Enter year and day-of-year: "); getstr(s);
      if (!s[0])
        break;
      strupr(s);
      gotoxy(2,row++);
      if (getint(s,&y,&day_num) != 2) {
        cprintf("year and day_of_year required");
        break;
      }
      if (y > 0 && strstr(s,"BC"))
        y = 1 - y;
      MaxDOY = year_size(y);
      if (y<MINYEAR || y>MAXYEAR || day_num > MaxDOY) {
        cprintf("Invalid year or day_of_year");
        break;
      }
      month_day(y,day_num,&m,&d_int);
      cprintf("%s %d\n",month_name[m],d_int);
      break;
    case 5:                  /* find day of week for given date */
      cprintf("Enter month, day, year: "); getstr(s);
      if (!s[0])
        break;
      gotoxy(2,row++);
      if (!read_date(s,&m,&d,&y)) {
        cprintf("Invalid entry");
        break;
      }
      d_int = (int)d;
      if(!valid(m, d_int, y)) {
        cprintf("Invalid date");
        break;
      }
      else {
        jul = julian_day(m,d,y);
        J = (long)jul;
        wk = J % 7;
        cprintf("Weekday : %s", day_name[wk]);
      }
      break;
    case 6:           /* find date of Easter for any year >= 325 */
      cprintf("Enter year: "); getstr(s);
      if (!s[0])
        break;
      y = atoi(s);
      if (y < 325 ) {
        gotoxy(2,row++);
        cprintf("The date of Easter was not set until 325 AD");
        break;
      }
      if (y > MAXYEAR) {
        gotoxy(2,row++);
        cprintf("Out of range");
        break;
      }
      day_num = easter(y)+80+leap(y);      /* easter returns the offset of
                                              Easter from March 21; Mar 21
                                              is day 80 in ordinary years,
                                              day 81 in leap years */
      month_day(y,day_num,&m,&d_int);
      gotoxy(2,row++);
      cprintf("Easter: %s %d\n",month_name[m],d_int);
      break;
    }
}

/*---------------------------------------------------------------------------*/
void high(int current)
{
  window(1,1,80,1);
  textattr(high_attr);
  gotoxy(col[current],1);
  cputs(msg[current]);
  textattr(norm_attr);
  window(left+1,top+1,right-1,bottom-1);
}

void norm(int current)
{
  window(1,1,80,1);
  textattr(bar_attr);
  gotoxy(col[current],1);
  cputs(msg[current]);
  textattr(norm_attr);
  window(left+1,top+1,right-1,bottom-1);
}

/*---------------------------------------------------------------------------*/
int initialize(int col[], int num)
{
  int i, total_length = 0, space;
  for (i=0; i<num; i++)
    total_length += strlen(msg[i]);
  if (total_length > 70)
    return 0;
  else {
    space = (75 - total_length)/(num-1); /* allowable space between entries */
    col[0] = 5;
    for (i=1; i<num; i++)
      col[i] = col[i-1] + space + strlen(msg[i-1]);
  }
  return 1;
}

/*---------------------------------------------------------------------------*/
int valid(int m, int d, int y) /* return 1 if m/d/y is a valid date, 0 if not */
{
   if(y < MINYEAR || y > MAXYEAR || m < 1 || m > 12
         || (y == 1582 && d>4 && d<15) ||
         d < 1 || d > days_in_month(m,y) +10*(y == 1582 && m == 10))
          /* since Oct, 1582 has only 21 days but the date entered
             may be as many as 31, add 10 to days_in_month() here */
    return 0;
  return 1;
}
/*---------------------------------------------------------------------------*/
/* decipher user input for date */
read_date(char *s, int *month, double *day, int *year)
{
  static char *short_name[] =
  {"JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG","SEP","OCT","NOV","DEC"};
  int cnt;
  double a,b,c;    /* entry numbers; month and year will be casted as ints */
  int i,y;         /* y is temporary value for *year */

  strupr(s);
  cnt = getdouble(s,&a,&b,&c);
  if (cnt == 3) {
    *month = (int)a;
    *day = b;
    y = (int)c;
    if (y > 0 && strstr(s,"BC"))
      y = 1 - y;
    *year = y;
    return 1;
  }
  else
    if (cnt == 2) {
    /* if only two numbers in s, month name must be included */
      for (i=0; i<12; i++) {
        if (strstr(s,short_name[i])) {
          *month = i+1;
          *day = a;
          y = (int)b;
          if (y > 0 && strstr(s,"BC"))
            y = 1 - y;
          *year = y;
          return 1;
        }
      }
    }
    else
      return 0;
}

void std_screen()            /* set up entry screen */
{
  window(1,1,80,25);
  textattr(7);
  clrscr();
  menu_bar(col,7);
  textattr(norm_attr);
  rect(left,top,right,bottom,norm_attr,1);
  window(left+1,top+1,right-1,bottom-1);
  clrscr();
}

void menu_bar(int *col, int n)
{
  int i;

  window(1,1,80,1);
  textattr(bar_attr);
  clrscr();
  if (!fill(col, 7)) {
    window(1,1,80,25);
    printf("Can't fill col[] array\n");
    getch();
    normal_exit();
  }
  for (i=0; i<n; i++) {
    gotoxy(col[i],1);
    cprintf(msg[i]);
  }
}

fill(int *col, int n)
{
  int i, total_length = 0, space;
  for (i=0; i<n; i++)
    total_length += strlen(msg[i]);
  if (total_length > 70)
    return 0;
  else {
    space = (75 - total_length)/(n-1);    /* space between entries */
    col[0] = 5;
    for (i=1; i<n; i++)
      col[i] = col[i-1] + space + strlen(msg[i-1]);
  }
  return 1;
}

void normal_exit()
{
  window(1,1,80,25);
  textattr(old_attr);                /* reset original attribute */
  clrscr();
  showcursor();
  exit(0);
}

